@@ -26,8 +26,10 @@ use crate::player::gst::GSTPlayer;
 use anyhow::{anyhow, Result};
 use humantime::{format_duration, FormattedDuration};
 use id3::frame::Lyrics;
-use id3::frame::Picture;
+use id3::frame::{Picture, PictureType};
 use id3::{Tag, Version};
+use mp4ameta::{Img, ImgFmt};
+use std::ffi::OsStr;
 use std::fs::rename;
 use std::path::Path;
 use std::str::FromStr;
@@ -90,128 +92,151 @@ impl Song {
     }
 
     pub fn save(&self) -> Result<()> {
-        match self.ext.as_ref().unwrap().as_str() {
-            "mp3" => {
-                let mut id3_tag = Tag::read_from_path(self.file.as_ref().unwrap())?;
-                id3_tag.set_artist(
-                    self.artist
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Artist")),
-                );
-                id3_tag.set_title(
-                    self.title
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Title")),
-                );
-                id3_tag.set_album(
-                    self.album
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Album")),
-                );
-                id3_tag.remove_all_lyrics();
+        if let Some(ext) = self.ext.as_ref() {
+            match ext.as_str() {
+                "mp3" => {
+                    let mut id3_tag = Tag::default();
+                    if let Some(file) = &self.file {
+                        if let Ok(t) = Tag::read_from_path(file) {
+                            id3_tag = t;
+                        }
+                    }
 
-                if let Some(mut lyric) = self.parsed_lyric.clone() {
-                    if let Some(text) = lyric.as_lrc() {
-                        let lyric_frame: Lyrics = Lyrics {
-                            lang: String::from("chi"),
-                            description: String::from("saved by termusic."),
-                            text,
-                        };
-                        id3_tag.add_lyrics(lyric_frame);
+                    id3_tag.set_artist(
+                        self.artist
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Artist")),
+                    );
+                    id3_tag.set_title(
+                        self.title
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Title")),
+                    );
+                    id3_tag.set_album(
+                        self.album
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Album")),
+                    );
+                    id3_tag.remove_all_lyrics();
+
+                    if let Some(mut lyric) = self.parsed_lyric.clone() {
+                        if let Some(text) = lyric.as_lrc() {
+                            let lyric_frame = Lyrics {
+                                lang: String::from("chi"),
+                                description: String::from("saved by termusic."),
+                                text,
+                            };
+                            id3_tag.add_lyrics(lyric_frame);
+                        }
+                    }
+
+                    for p in self.picture.iter() {
+                        id3_tag.add_picture(p.clone());
+                    }
+
+                    if let Some(file) = self.file.as_ref() {
+                        id3_tag.write_to_path(file, Version::Id3v24)?;
                     }
                 }
 
-                if let Some(file) = self.file.as_ref() {
-                    id3_tag.write_to_path(file, Version::Id3v24)?;
+                "m4a" => {
+                    let mut m4a_tag = mp4ameta::Tag::default();
+                    if let Some(file) = &self.file {
+                        if let Ok(t) = mp4ameta::Tag::read_from_path(file) {
+                            m4a_tag = t;
+                        }
+                    }
+
+                    m4a_tag.set_artist(
+                        self.artist
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Artist")),
+                    );
+                    m4a_tag.set_title(
+                        self.title
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Title")),
+                    );
+                    m4a_tag.set_album(
+                        self.album
+                            .as_ref()
+                            .unwrap_or(&String::from("Unknown Album")),
+                    );
+                    m4a_tag.remove_lyrics();
+
+                    if let Some(mut lyric) = self.parsed_lyric.clone() {
+                        if let Some(text) = lyric.as_lrc() {
+                            m4a_tag.set_lyrics(text);
+                        }
+                    }
+
+                    for p in self.picture.iter() {
+                        let img = Img {
+                            data: p.data.clone(),
+                            fmt: ImgFmt::Jpeg,
+                        };
+                        m4a_tag.set_artwork(img);
+                    }
+
+                    if let Some(file) = self.file.as_ref() {
+                        // m4a_tag.write_to_path(file)?;
+                        let _ = m4a_tag
+                            .write_to_path(file)
+                            .map_err(|e| anyhow!("write m4a tag error {:?}", e))?;
+                    }
                 }
 
-                Ok(())
+                &_ => {}
             }
-            "m4a" => {
-                // let mut m4a_tag: mp4ameta::Tag =
-                //     match mp4ameta::Tag::read_from_path(self.file.as_ref().unwrap()) {
-                //         Ok(t) => t,
-                //         Err(_) => mp4ameta::Tag::default(),
-                //     };
-
-                let mut m4a_tag = mp4ameta::Tag::read_from_path(self.file.as_ref().unwrap())?;
-                m4a_tag.set_artist(
-                    self.artist
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Artist")),
-                );
-                m4a_tag.set_title(
-                    self.title
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Title")),
-                );
-                m4a_tag.set_album(
-                    self.album
-                        .as_ref()
-                        .unwrap_or(&String::from("Unknown Album")),
-                );
-                // m4a_tag.remove_lyrics();
-
-                // if let Some(mut lyric) = self.parsed_lyric.clone() {
-                //     if let Some(text) = lyric.as_lrc() {
-                //         m4a_tag.set_lyrics(text);
-                //     }
-                // }
-
-                if let Some(file) = self.file.as_ref() {
-                    // m4a_tag.write_to_path(file)?;
-                    let _ = m4a_tag
-                        .write_to_path(file)
-                        .map_err(|e| anyhow!("write m4a tag error {:?}", e));
-                }
-
-                Ok(())
-            }
-
-            &_ => Ok(()),
         }
+
+        Ok(())
     }
 
     pub fn rename_by_tag(&mut self) -> Result<()> {
         let new_name = format!(
             "{}-{}.{}",
-            self.artist.as_ref().unwrap(),
-            self.title.as_ref().unwrap(),
-            self.ext.as_ref().unwrap(),
+            self.artist().unwrap_or(&"Unknown Artist".to_string()),
+            self.title().unwrap_or(&"Unknown Title".to_string()),
+            self.ext.as_ref().unwrap_or(&"mp3".to_string()),
         );
         let new_name_path: &Path = Path::new(new_name.as_str());
-        let p_old: &Path = Path::new(self.file.as_ref().unwrap());
-        let p_prefix = p_old.parent().unwrap();
-        let p_new = p_prefix.join(new_name_path);
-        rename(p_old, p_new.clone())?;
-        self.file = Some(String::from(p_new.to_string_lossy()));
-        self.name = Some(String::from(p_new.file_name().unwrap().to_string_lossy()));
+        if let Some(file) = &self.file {
+            let p_old: &Path = Path::new(file);
+            if let Some(p_prefix) = p_old.parent() {
+                let p_new = p_prefix.join(new_name_path);
+                rename(p_old, p_new.clone())
+                    .map_err(|e| anyhow!("rename m4a file error {:?}", e))?;
+                self.file = Some(String::from(p_new.to_string_lossy()));
+                if let Some(name) = p_new.file_name() {
+                    self.name = Some(String::from(name.to_string_lossy()));
+                }
+            }
+        }
         Ok(())
     }
 
     pub fn set_lyric(&mut self, lyric_string: String, lang_ext: String) {
-        match self.ext.as_ref().unwrap().as_str() {
-            "mp3" => {
-                self.lyric_frames.clear();
-                self.lyric_frames.push(Lyrics {
-                    lang: lang_ext,
-                    description: String::from("added by termusic."),
-                    text: lyric_string,
-                });
+        self.lyric_frames.clear();
+        self.lyric_frames.push(Lyrics {
+            lang: lang_ext,
+            description: String::from("added by termusic."),
+            text: lyric_string,
+        });
 
-                let mut parsed_lyric: Option<Lyric> = None;
-                if !self.lyric_frames.is_empty() {
-                    parsed_lyric = match Lyric::from_str(self.lyric_frames[0].text.as_ref()) {
-                        Ok(l) => Some(l),
-                        Err(_) => None,
-                    };
-                }
-                self.parsed_lyric = parsed_lyric;
-            }
-            "m4a" => {}
-
-            &_ => {}
+        let mut parsed_lyric: Option<Lyric> = None;
+        if !self.lyric_frames.is_empty() {
+            parsed_lyric = match Lyric::from_str(self.lyric_frames[0].text.as_ref()) {
+                Ok(l) => Some(l),
+                Err(_) => None,
+            };
         }
+        self.parsed_lyric = parsed_lyric;
+    }
+
+    pub fn set_photo(&mut self, picture: Picture) {
+        self.picture.clear();
+        self.picture.push(picture);
     }
 }
 
@@ -220,10 +245,11 @@ impl FromStr for Song {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let p: &Path = Path::new(s);
-        let ext = String::from(p.extension().unwrap().to_string_lossy());
-        match ext.as_str() {
-            "mp3" => {
-                let name = Some(String::from(p.file_name().unwrap().to_string_lossy()));
+        let ext = p.extension().and_then(OsStr::to_str);
+
+        match ext {
+            Some("mp3") => {
+                let name = p.file_name().and_then(OsStr::to_str).map(|x| x.to_string());
                 let duration: Option<Duration> = match mp3_duration::from_path(s) {
                     Ok(d) => Some(d),
                     Err(_) => Some(Duration::from_secs(0)),
@@ -272,14 +298,14 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: Some(ext),
+                    ext: Some("mp3".to_string()),
                     lyric_frames: lyrics,
                     parsed_lyric,
                     picture,
                 })
             }
-            "m4a" => {
-                let name = Some(String::from(p.file_name().unwrap().to_string_lossy()));
+            Some("m4a") => {
+                let name = p.file_name().and_then(OsStr::to_str).map(|x| x.to_string());
 
                 let duration_u64 = GSTPlayer::duration_m4a(s);
                 let duration = Some(Duration::from_secs(duration_u64));
@@ -298,18 +324,13 @@ impl FromStr for Song {
                     }
                 };
 
-                // println!("{}", tag.artist().unwrap());
-
                 let artist: Option<String> = m4a_tag.artist().map(String::from);
                 let album: Option<String> = m4a_tag.album().map(String::from);
                 let title: Option<String> = m4a_tag.title().map(String::from);
-                // for l in m4a_tag.lyrics().cloned() {
-                //     lyrics.push(l);
-                // }
-                let lyrics = m4a_tag.lyrics().map(String::from);
 
+                let lyrics = m4a_tag.lyrics().map(String::from);
                 let parsed_lyric: Option<Lyric>;
-                match lyrics {
+                match lyrics.clone() {
                     Some(s) => {
                         parsed_lyric = match Lyric::from_str(&s) {
                             Ok(l) => Some(l),
@@ -319,11 +340,29 @@ impl FromStr for Song {
                     None => parsed_lyric = None,
                 }
 
-                let lyrics: Vec<Lyrics> = Vec::new();
-                let picture: Vec<Picture> = Vec::new();
-                // for p in m4a_tag.artworks() {
-                //     picture.push(p);
-                // }
+                let mut lyric_frames: Vec<Lyrics> = Vec::new();
+                if let Some(s) = lyrics {
+                    lyric_frames.push(Lyrics {
+                        lang: String::from("chi"),
+                        description: String::from("saved by termusic."),
+                        text: s,
+                    });
+                };
+
+                let mut picture: Vec<Picture> = Vec::new();
+                if let Some(artwork) = m4a_tag.artwork() {
+                    let fmt = match artwork.fmt {
+                        ImgFmt::Bmp => "image/bmp",
+                        ImgFmt::Jpeg => "image/jpeg",
+                        ImgFmt::Png => "image/png",
+                    };
+                    picture.push(Picture {
+                        mime_type: fmt.to_string(),
+                        picture_type: PictureType::Other,
+                        description: "some image".to_string(),
+                        data: artwork.data.to_vec(),
+                    });
+                }
 
                 let file = Some(String::from(s));
                 Ok(Self {
@@ -333,8 +372,8 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: Some(ext),
-                    lyric_frames: lyrics,
+                    ext: Some("m4a".to_string()),
+                    lyric_frames,
                     parsed_lyric,
                     picture,
                 })
@@ -356,7 +395,7 @@ impl FromStr for Song {
                     file,
                     duration,
                     name,
-                    ext: Some(ext),
+                    ext: ext.map(|x| x.to_string()),
                     lyric_frames: lyrics,
                     parsed_lyric,
                     picture,
