@@ -34,7 +34,6 @@ use std::str::FromStr;
 use humantime::format_duration;
 // use lrc::{Lyrics, TimeTag};
 use super::TransferState;
-use crate::invidious::InvidiousInstance;
 use crate::player::AudioPlayer;
 use std::path::{Path, PathBuf};
 use std::time::Duration;
@@ -81,14 +80,15 @@ impl MainActivity {
                     // Update tree
                     self.scan_dir(PathBuf::from(node_id.as_str()).as_path());
                     // Update
-                    let props = TreeViewPropsBuilder::from(
-                        self.view.get_props(COMPONENT_TREEVIEW).unwrap(),
-                    )
-                    .with_tree(self.tree.root())
-                    .with_title(String::from(self.path.to_string_lossy()),Alignment::Left)
-                    .build();
-                    let msg = self.view.update(COMPONENT_TREEVIEW, props);
-                    self.update(msg)
+                    if let Some(props) = self.view.get_props(COMPONENT_TREEVIEW) {
+                        let props = TreeViewPropsBuilder::from(props)
+                            .with_tree(self.tree.root())
+                            .with_title(String::from(self.path.to_string_lossy()),Alignment::Left)
+                            .build();
+                        let msg = self.view.update(COMPONENT_TREEVIEW, props);
+                        self.update(msg);
+                    }
+                    None
                 }
                 (COMPONENT_TREEVIEW, key) if key== &MSG_KEY_BACKSPACE => {
                     // Update tree
@@ -98,14 +98,15 @@ impl MainActivity {
                             let p: PathBuf = p.to_path_buf();
                             self.scan_dir(p.as_path());
                             // Update
-                            let props = TreeViewPropsBuilder::from(
-                                self.view.get_props(COMPONENT_TREEVIEW).unwrap(),
-                            )
-                            .with_tree(self.tree.root())
-                            .with_title(String::from(self.path.to_string_lossy()),Alignment::Left)
-                            .build();
-                            let msg = self.view.update(COMPONENT_TREEVIEW, props);
-                            self.update(msg)
+                            if let Some(props) = self.view.get_props(COMPONENT_TREEVIEW) {
+                                let props = TreeViewPropsBuilder::from(props)
+                                    .with_tree(self.tree.root())
+                                    .with_title(String::from(self.path.to_string_lossy()),Alignment::Left)
+                                    .build();
+                                let msg = self.view.update(COMPONENT_TREEVIEW, props);
+                                self.update(msg);
+                            }
+                            None
                         }
                     }
                 }
@@ -329,9 +330,8 @@ impl MainActivity {
                         } else {
                             self.mount_youtube_options();
                             self.youtube_options_url = url.clone();
-                            let domain = self.config.invidious_instance.clone();
-                            let mut inv = InvidiousInstance::new(domain);
-                            match inv.get_search_query(url,1) {
+                            self.invidious_instance = crate::invidious::InvidiousInstance::new();
+                            match self.invidious_instance.get_search_query(url,1) {
                                 Ok(y) => {
                                     self.youtube_options = y;
                                     self.youtube_options_index = 1;
@@ -344,10 +344,8 @@ impl MainActivity {
                 }
 
                 (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if key== &MSG_KEY_TAB => {
-                    let domain = self.config.invidious_instance.clone();
-                    let mut inv = InvidiousInstance::new(domain);
                     self.youtube_options_index +=1;
-                    match inv.get_search_query(self.youtube_options_url.as_str(),self.youtube_options_index) {
+                    match self.invidious_instance.get_search_query(self.youtube_options_url.as_str(),self.youtube_options_index) {
                         Ok(y) => {
                             self.youtube_options = y;
                             self.sync_youtube_options(self.youtube_options_index);
@@ -358,11 +356,9 @@ impl MainActivity {
                 }
 
                 (super::COMPONENT_SCROLLTABLE_YOUTUBE,key) if key== &MSG_KEY_SHIFT_TAB => {
-                    let domain = self.config.invidious_instance.clone();
-                    let mut inv = InvidiousInstance::new(domain);
                     if self.youtube_options_index >1 {
                     self.youtube_options_index -=1;
-                    match inv.get_search_query(self.youtube_options_url.as_str(),self.youtube_options_index) {
+                    match self.invidious_instance.get_search_query(self.youtube_options_url.as_str(),self.youtube_options_index) {
                         Ok(y) => {
                             self.youtube_options = y;
                             self.sync_youtube_options(self.youtube_options_index);
@@ -465,12 +461,12 @@ impl MainActivity {
                     None
                 }
                 // -- help
-                (COMPONENT_TEXT_HELP,key) if (key==  &MSG_KEY_ENTER) | (key == &MSG_KEY_ESC) => {
+                (COMPONENT_TEXT_HELP,key) if (key==  &MSG_KEY_ENTER) | (key == &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q) => {
                     self.umount_help();
                     None
                 }
                 // -- error
-                (COMPONENT_TEXT_ERROR,key) if (key==  &MSG_KEY_ESC) | (key == &MSG_KEY_ENTER) => {
+                (COMPONENT_TEXT_ERROR,key) if (key==  &MSG_KEY_ESC) | (key == &MSG_KEY_ENTER) | (key == &MSG_KEY_CHAR_CAPITAL_Q)=> {
                     self.umount_error();
                     None
                 }
@@ -507,7 +503,7 @@ impl MainActivity {
             return;
         }
 
-        let song = match self.current_song.as_ref() {
+        let song = match self.current_song.clone() {
             Some(song) => song,
             None => return,
         };
@@ -517,21 +513,24 @@ impl MainActivity {
 
         if time_pos > self.time_pos || time_pos < 2 {
             self.time_pos = time_pos;
-            let props = self.view.get_props(COMPONENT_PROGRESS).unwrap();
-            let props = progress_bar::ProgressBarPropsBuilder::from(props)
-                .with_progress(new_prog)
-                .with_title(
-                    format!("Playing: {} - {}", artist, title),
-                    Alignment::Center,
-                )
-                .with_label(format!(
-                    "{}     :     {} ",
-                    format_duration(Duration::from_secs(time_pos as u64)),
-                    format_duration(Duration::from_secs(duration as u64))
-                ))
-                .build();
-            self.view.update(COMPONENT_PROGRESS, props);
-            self.redraw = true;
+            if let Some(props) = self.view.get_props(COMPONENT_PROGRESS) {
+                let props = progress_bar::ProgressBarPropsBuilder::from(props)
+                    .with_progress(new_prog)
+                    .with_title(
+                        format!("Playing: {} - {}", artist, title),
+                        Alignment::Center,
+                    )
+                    .with_label(format!(
+                        "{}     :     {} ",
+                        format_duration(Duration::from_secs(time_pos as u64)),
+                        format_duration(Duration::from_secs(duration as u64))
+                    ))
+                    .build();
+                self.view.update(COMPONENT_PROGRESS, props);
+                self.redraw = true;
+                self.view();
+                self.redraw = false;
+            }
         }
 
         // Update lyrics
@@ -540,31 +539,33 @@ impl MainActivity {
         }
 
         if song.lyric_frames.is_empty() {
-            let props = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC).unwrap();
-            let props = paragraph::ParagraphPropsBuilder::from(props)
-                .with_title("Lyrics", Alignment::Left)
-                .with_texts(vec![TextSpan::new("No lyrics available.")])
-                .build();
-            self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
-            return;
+            if let Some(props) = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC) {
+                let props = paragraph::ParagraphPropsBuilder::from(props)
+                    .with_title("Lyrics", Alignment::Left)
+                    .with_texts(vec![TextSpan::new("No lyrics available.")])
+                    .build();
+                self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
+                return;
+            }
         }
 
-        let line = match song.parsed_lyric.as_ref() {
-            Some(l) => {
-                if l.unsynced_captions.is_empty() {
-                    return;
-                }
-                l.get_text(time_pos).unwrap()
+        let mut line = String::new();
+        if let Some(l) = song.parsed_lyric.as_ref() {
+            if l.unsynced_captions.is_empty() {
+                return;
             }
-            None => String::from(""),
-        };
+            if let Some(l) = l.get_text(time_pos) {
+                line = l;
+            }
+        }
 
-        let props = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC).unwrap();
-        let props = paragraph::ParagraphPropsBuilder::from(props)
-            .with_title("Lyrics", Alignment::Left)
-            .with_texts(vec![TextSpan::new(line)])
-            .build();
-        self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
+        if let Some(props) = self.view.get_props(COMPONENT_PARAGRAPH_LYRIC) {
+            let props = paragraph::ParagraphPropsBuilder::from(props)
+                .with_title("Lyrics", Alignment::Left)
+                .with_texts(vec![TextSpan::new(line)])
+                .build();
+            self.view.update(COMPONENT_PARAGRAPH_LYRIC, props);
+        }
     }
 
     // update picture of album
@@ -580,8 +581,10 @@ impl MainActivity {
         };
 
         // clear all previous image
-        self.context.as_mut().unwrap().clear_image();
-
+        match self.context.as_mut() {
+            Some(c) => c.clear_image(),
+            None => return,
+        }
         // if no photo, just return
         if song.picture.is_empty() {
             return;
@@ -606,7 +609,7 @@ impl MainActivity {
                 height: None,
                 ..Default::default()
             };
-            viuer::print(&image, &config).expect("image printing failed.");
+            let _ = viuer::print(&image, &config);
         };
     }
 
@@ -636,30 +639,30 @@ impl MainActivity {
             true => {
                 let text = "Press \"?\" for help.".to_string();
 
-                let props = label::LabelPropsBuilder::from(
-                    self.view.get_props(super::COMPONENT_LABEL_HELP).unwrap(),
-                )
-                .with_text(text)
-                .with_background(tuirealm::tui::style::Color::Reset)
-                .with_foreground(tuirealm::tui::style::Color::Cyan)
-                .build();
+                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
+                    let props = label::LabelPropsBuilder::from(props)
+                        .with_text(text)
+                        .with_background(tuirealm::tui::style::Color::Reset)
+                        .with_foreground(tuirealm::tui::style::Color::Cyan)
+                        .build();
 
-                let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
-                self.update(msg);
+                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    self.update(msg);
+                }
             }
             false => {
                 let text = " Downloading...".to_string();
 
-                let props = label::LabelPropsBuilder::from(
-                    self.view.get_props(super::COMPONENT_LABEL_HELP).unwrap(),
-                )
-                .with_text(text)
-                .with_foreground(tuirealm::tui::style::Color::White)
-                .with_background(tuirealm::tui::style::Color::Red)
-                .build();
+                if let Some(props) = self.view.get_props(super::COMPONENT_LABEL_HELP) {
+                    let props = label::LabelPropsBuilder::from(props)
+                        .with_text(text)
+                        .with_foreground(tuirealm::tui::style::Color::White)
+                        .with_background(tuirealm::tui::style::Color::Red)
+                        .build();
 
-                let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
-                self.update(msg);
+                    let msg = self.view.update(super::COMPONENT_LABEL_HELP, props);
+                    self.update(msg);
+                }
             }
         }
     }
